import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.extensions.java6.auth.oauth2.AuthorizationCodeInstalledApp;
import com.google.api.client.extensions.jetty.auth.oauth2.LocalServerReceiver;
import com.google.api.client.googleapis.auth.oauth2.GoogleAuthorizationCodeFlow;
import com.google.api.client.googleapis.auth.oauth2.GoogleClientSecrets;
import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.store.FileDataStoreFactory;
import com.google.api.services.drive.Drive;
import com.google.api.services.drive.DriveScopes;
import com.google.api.services.drive.model.File;
import com.google.api.services.drive.model.FileList;

import java.io.*;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Date;
import java.text.*;

public class GDriveReport {

    static PrintWriter olog;
    static {
    	try {
			olog = new PrintWriter(new FileWriter("GoogleDriveReport.log"));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

    private static final String APPLICATION_NAME = "GDriveReport";
    private static final java.io.File DATA_STORE_DIR = new java.io.File(System.getProperty("user.home"), ".");
    private static FileDataStoreFactory dataStoreFactory;
    private static HttpTransport httpTransport;
    private static final JsonFactory JSON_FACTORY = JacksonFactory.getDefaultInstance();
    private static Drive drive;
    private static final List<String> SCOPES = Arrays.asList("https://www.googleapis.com/auth/drive");

    private static Credential authorize() throws Exception {

        // load client secrets
        GoogleClientSecrets clientSecrets = GoogleClientSecrets.load(JSON_FACTORY,
                new InputStreamReader(GDriveReport.class.getResourceAsStream("client_secret.json")));

        // set up authorization code flow
        GoogleAuthorizationCodeFlow flow = new GoogleAuthorizationCodeFlow.Builder(
                httpTransport, JSON_FACTORY, clientSecrets,
                Collections.singleton(DriveScopes.DRIVE_FILE)).setDataStoreFactory(dataStoreFactory)
                .setScopes(SCOPES)
                .build();

        // authorize
        return new AuthorizationCodeInstalledApp(flow, new LocalServerReceiver()).authorize("user");
    }

    public static void main(String[] args) {
        try {
            httpTransport = GoogleNetHttpTransport.newTrustedTransport();
            dataStoreFactory = new FileDataStoreFactory(DATA_STORE_DIR);
            Credential credential = authorize();
            drive = new Drive.Builder(httpTransport, JSON_FACTORY, credential).setApplicationName(APPLICATION_NAME).build();

			olog.println("List of Google Drive Backups");
			olog.println("============================");
			String DateStamp = new SimpleDateFormat("MMMMM d, yyyy (EEEE) HH:mm:ss").format(new Date());
			olog.println(DateStamp);
			olog.println();

			long bytesUsed = 0;
			int fileCnt=0;
			FileList l = drive.files().list().setMaxResults(200).execute();
			List<File> files = l.getItems();
			if (files == null || files.size() == 0) {
				olog.println("No files found.");
			} else {
				olog.println("    File Name                     MBytes  In Folder        File ID");
				olog.println("    ----------------------------  ------  ---------------  ----------------------------");

				for (File file : files) {
					fileCnt++;
					String item = String.format("%2d  ",(fileCnt));

					String fileName = file.getTitle();
					String name = fileName + "                              ";
					name = name.substring(0,30);

					long fileSize = 0;
					if(file.getFileSize() != null)fileSize = file.getFileSize();
					bytesUsed = bytesUsed + fileSize;
					String arcMbytes = String.format("%6d  ",(fileSize/1024/1024));

					String folder = "[Others Shared]  ";
					String fileFolder = "[Others Shared]  ";
					boolean trashed = file.getLabels().getTrashed();
					if(trashed)folder="[Trash folder]   ";

					boolean sharedWithMe = false;
					if(file.getParents().size() == 0)sharedWithMe = true;

					String parentId = "No folder information";
					if(!sharedWithMe){
						parentId = file.getParents().get(0).getId();
						File folderFile = drive.files().get(parentId).execute();
						fileFolder = folderFile.getTitle();
						folder = fileFolder + "                    ";
						folder = folder.substring(0,17);
					}
					olog.println(item +name +arcMbytes +folder + file.getId());
//					olog.println("                                              ...folderID  " +parentId);

				}
				String totalMbytes = String.format("%7d  ",(bytesUsed/1024/1024));
				olog.println("                                  ------");
				olog.println("Total MB in archive              " +totalMbytes);
			}
        } catch (IOException e) {
            System.err.println(e.getMessage());
        } catch (Throwable t) {
            t.printStackTrace();
        }
        olog.close();
    }
}